package hikapro.com.backpack;

import android.app.Fragment;
import android.app.FragmentManager;
import android.os.Bundle;
import android.util.Log;

import java.lang.ref.WeakReference;
import java.util.HashMap;

/**
 * Created by tariel on 24/04/16.
 */
public class StateMaintainer {

    protected final String TAG = getClass().getSimpleName();
    private final String stateMaintainerTag;
    private final WeakReference<FragmentManager> fragmentManager;
    private StateMngFragment stateMngFragment;
    private boolean isRecreating;


    public StateMaintainer(FragmentManager fragmentManager, String stateMaintainerTAG) {
        this.fragmentManager = new WeakReference<>(fragmentManager);
        this.stateMaintainerTag = stateMaintainerTAG;
    }

    public boolean init() {
        try {
            stateMngFragment = (StateMngFragment)
                    fragmentManager.get().findFragmentByTag(stateMaintainerTag);

            if (stateMngFragment == null) {
                stateMngFragment = new StateMngFragment();
                fragmentManager.get().beginTransaction()
                        .add(stateMngFragment, stateMaintainerTag).commit();
                isRecreating = false;
                return true;
            } else {
                isRecreating = true;
                return false;
            }
        } catch (NullPointerException e) {
            return false;
        }
    }

    public boolean wasRecreated() { return isRecreating; }

    public void put(String key, Object obj) {
        stateMngFragment.put(key, obj);
    }

    public void put(Object obj) {
        put(obj.getClass().getName(), obj);
    }

    public <T> T get(String key)  {
        return stateMngFragment.get(key);

    }

    public boolean hasKey(String key) {
        return stateMngFragment.get(key) != null;
    }


    public static class StateMngFragment extends Fragment {

        private HashMap<String, Object> data = new HashMap<>();

        @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            // Grants that the fragment will be preserved
            setRetainInstance(true);
        }


        public void put(String key, Object obj) {
            data.put(key, obj);
            Log.i(this.toString(), String.format("Put object %s Total count %d", key, data.size()));
        }


        public void put(Object object) {
            put(object.getClass().getName(), object);
        }


        @SuppressWarnings("unchecked")
        public <T> T get(String key) {
            Log.i(this.toString(), String.format("Get object %s Total count %d", key, data.size()));
            return (T) data.get(key);
        }
    }
}
