package hikapro.com.backpack.model;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Hashtable;
import java.util.List;

import hikapro.com.backpack.model.entities.Category;
import hikapro.com.backpack.model.entities.Item;
import hikapro.com.backpack.presenter.Presenter;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * Created by tariel on 22/04/16.
 */
public class ItemModel implements Model.Item {

    private Api                 api;
    private Presenter.ItemList  presenter;
    private List<Category>      rawCategories;
    private List<Category>      sortedCategories;
    private List<Item>          rawItems;

    private Hashtable<Category, List<Item>> items;

    public ItemModel() {
        this.api                = RestClient.getApi();
        this.rawCategories      = new ArrayList<>();
        this.rawItems           = new ArrayList<>();
        this.sortedCategories   = new ArrayList<>();
    }

    // categories -->

    @Override
    public Category getCategoryByPosition(int position) {
        return sortedCategories.get(position);
    }
    @Override
    public int getCategoriesCount() {
        return sortedCategories.size();
    }

    // categories <--

    // items -->

    @Override
    public int insertItem(Item item) {
        return 0;
    }
    @Override
    public boolean deleteItem(int id) {
        return false;
    }
    @Override
    public Item findItem(int id) {
        Item item = null;
        for (Item i : rawItems) {
            if (i.getId() == id) {
                item = i;
                break;
            }
        }
        return item;
    }
    @Override
    public Item getItemByPosition(int categoryId, int position) {
        Item ret = null;
        Category category = findSortedCategory(categoryId);
        if (category != null) {
            ret = items.get(category).get(position);
        }
        return ret;
    }
    @Override
    public int getItemsCount(int categoryId) {
        int ret = 0;
        Category category = findSortedCategory(categoryId);
        if (category != null) {
            ret = items.get(category).size();
        }
        return ret;
    }
    // items <--

    // events -->

    @Override
    public void notifyDataSetChanged() {
        presenter.notifyDataSetChanged();
    }

    @Override
    public void onDestroy(boolean isConfigurationChanging) {
        if ( !isConfigurationChanging ) {
            presenter = null;
        }
    }
    @Override
    public void sendMessage(String message) {
        presenter.showMessage(message);
    }
    // events <--

    // process -->

    @Override
    public void executeQuery() {
        if (rawCategories.isEmpty() || rawItems.isEmpty()) {
            loadCategories();
            loadItems();
        } else {
            notifyDataSetChanged();
        }

    }
    private void loadCategories() {
        Call<List<Category>> call = api.getItemCategories();
        call.enqueue(new Callback<List<Category>>() {
            @Override
            public void onResponse(Call<List<Category>> call, Response<List<Category>> response) {
                int statusCode = response.code();
                rawCategories = response.body();
            }
            @Override
            public void onFailure(Call<List<Category>> call, Throwable t) {

            }
        });
    }
    private void loadItems() {
        Call<List<Item>> call2 = api.getItems();
        call2.enqueue(new Callback<List<Item>>() {
            @Override
            public void onResponse(Call<List<Item>> call, Response<List<Item>> response) {
                int statusCode = response.code();
                rawItems = response.body();
                initData();
            }
            @Override
            public void onFailure(Call<List<Item>> call, Throwable t) {

            }
        });
    }
    private void syncData() {
        // TODO sync data here
    }
    // process <--

    // other -->


    @Override
    public void setPresenter(Presenter.ItemList presenter) {
        this.presenter = presenter;
    }

    @Override
    public Presenter.ItemList getPresenter() {
        return presenter;
    }

    private void initData() {
        List<Integer> ids       = presenter.getCurrentSet().getItems();
        List<Item> sortedItems  = new ArrayList<>(ids.size());
        Category category;

        for (Item item : rawItems) {
            if (ids.contains(item.getId())) {
                sortedItems.add(item);
            }
        }
        Collections.sort(sortedItems);
        items = new Hashtable<>(sortedItems.size());

        for (Item item : sortedItems) {
            category = findCategory(item.getCategory());
            if (category != null) {
                if (items.containsKey(category)) {
                    items.get(category).add(item);
                } else {
                    List<Item> innerList = new ArrayList<>(20);
                    innerList.add(item);
                    items.put(category, innerList);
                }
            }
        }
        Category[] array = items.keySet().toArray(new Category[items.size()]);
        sortedCategories = Arrays.asList(array);
        notifyDataSetChanged();
    }
    private Category findSortedCategory(int categoryId) {
        Category category = null;
        for (Category c : sortedCategories) {
            if (c.getId() == categoryId) {
                category = c;
                break;
            }
        }
        return category;

    }
    private Category findCategory(int categoryId) {
        Category category = null;

        for (Category c : rawCategories) {
            if (c.getId() == categoryId) {
                category = c;
                break;
            }
        }
        return category;
    }
    // other <--
}
