package hikapro.com.backpack.model.database;

import android.content.ContentValues;
import android.database.Cursor;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

import java.lang.reflect.Type;
import java.util.List;

import hikapro.com.backpack.model.entities.Category;
import hikapro.com.backpack.model.entities.Item;
import hikapro.com.backpack.model.entities.Set;

/**
 * Created by tariel on 20/04/16.
 */
public class Db {

    public Db() {
    }

    public abstract static class CategoriesTable {

        public static final String TABLE_NAME = "CATEGORIES";

        public static final String COLUMN_ID = "_id";
        public static final String COLUMN_NAME = "NAME";

        public static final String CREATE =
                "CREATE TABLE " + TABLE_NAME + " (" +
                        COLUMN_ID + " INTEGER PRIMARY KEY, " +
                        COLUMN_NAME + " TEXT NOT NULL" +
                        " ); ";

        public static ContentValues toContentValues(Category category) {
            ContentValues values = new ContentValues();
            values.put(COLUMN_ID, category.getId());
            values.put(COLUMN_NAME, category.getName());
            return values;
        }

        public static Category parseCursor(Cursor cursor) {
            Category category = new Category();
            category.setId(cursor.getInt(cursor.getColumnIndexOrThrow(COLUMN_ID)));
            category.setName(cursor.getString(cursor.getColumnIndexOrThrow(COLUMN_NAME)));
            return category;
        }

    }

    public abstract static class ItemsTable {
        public static final String TABLE_NAME = "ITEMS";

        public static final String COLUMN_ID = "_id";
        public static final String COLUMN_NAME = "NAME";
        public static final String COLUMN_CATEGORY = "CATEGORY";
        public static final String COLUMN_DESCRIPTION = "DESCRIPTION";
        public static final String COLUMN_BUY_URLS = "BUY_URLS";
        public static final String COLUMN_PHOTO_URL = "PHOTO_URL";
        public static final String COLUMN_PHOTO_THUMB_URL = "PHOTO_THUMB_URL";
        public static final String COLUMN_PHOTO_LOCAL = "PHOTO_LOCAL";
        public static final String COLUMN_PHOTO_THUMB_LOCAL = "PHOTO_THUMB_LOCAL";


        public static final String CREATE =
                "CREATE TABLE " + TABLE_NAME + " (" +
                        COLUMN_ID + " INTEGER PRIMARY KEY, " +
                        COLUMN_NAME + " TEXT NOT NULL, " +
                        COLUMN_CATEGORY + " INTEGER NOT NULL, " +
                        COLUMN_DESCRIPTION + " TEXT, " +
                        COLUMN_BUY_URLS + " TEXT, " +
                        COLUMN_PHOTO_URL + " TEXT, " +
                        COLUMN_PHOTO_THUMB_URL + " TEXT, " +
                        COLUMN_PHOTO_LOCAL + " TEXT, " +
                        COLUMN_PHOTO_THUMB_LOCAL + " TEXT" +
                        " ); ";

        public static ContentValues toContentValues(Item item) {
            ContentValues values = new ContentValues();
            values.put(COLUMN_ID, item.getId());
            values.put(COLUMN_NAME, item.getName());
            values.put(COLUMN_CATEGORY, item.getCategory());
            if (item.getDescription() != null)
                values.put(COLUMN_DESCRIPTION, item.getDescription());
            if (item.getBuyUrls() != null && !item.getBuyUrls().isEmpty()) {
                Gson gson = new Gson();
                Type type = new TypeToken<List<String>>() {}.getType();
                String json = gson.toJson(item.getBuyUrls(), type);
                values.put(COLUMN_BUY_URLS, json);
            }
            if (item.getPhotoUrl() != null)
                values.put(COLUMN_PHOTO_URL, item.getPhotoUrl());
            if (item.getPhotoThumbUrl() != null)
                values.put(COLUMN_PHOTO_THUMB_URL, item.getPhotoThumbUrl());
            /*
            values.put(COLUMN_PHOTO_LOCAL, );
            values.put(COLUMN_PHOTO_THUMB_LOCAL, item.getName());
            */
            return values;
        }

        public static Item parseCursor(Cursor cursor) {
            Item item = new Item();
            item.setId(cursor.getInt(cursor.getColumnIndexOrThrow(COLUMN_ID)));
            item.setName(cursor.getString(cursor.getColumnIndexOrThrow(COLUMN_NAME)));

            item.setCategory(cursor.getInt(cursor.getColumnIndexOrThrow(COLUMN_CATEGORY)));

            item.setDescription(cursor.getString(cursor.getColumnIndexOrThrow(COLUMN_DESCRIPTION)));
            String urls = cursor.getString(cursor.getColumnIndexOrThrow(COLUMN_BUY_URLS));
            if (urls != null && !urls.isEmpty()) {
                Gson gson = new Gson();
                Type type = new TypeToken<List<String>>() {}.getType();
                List<String> fromJson = gson.fromJson(urls, type);
                item.setBuyUrls(fromJson);
            }
            item.setPhotoUrl(cursor.getString(cursor.getColumnIndexOrThrow(COLUMN_PHOTO_URL)));
            item.setPhotoThumbUrl(cursor.getString(cursor.getColumnIndexOrThrow(COLUMN_PHOTO_THUMB_URL)));
            return item;
        }

    }

    public abstract static class SetsTable {
        public static final String TABLE_NAME = "SETS";

        public static final String COLUMN_ID = "_id";
        public static final String COLUMN_NAME = "NAME";
        public static final String COLUMN_ITEMS = "ITEMS";
        public static final String COLUMN_PHOTO_URL = "PHOTO_URL";
        public static final String COLUMN_PHOTO_THUMB_URL = "PHOTO_THUMB_URL";
        public static final String COLUMN_PHOTO_LOCAL = "PHOTO_LOCAL";
        public static final String COLUMN_PHOTO_THUMB_LOCAL = "PHOTO_THUMB_LOCAL";
        public static final String COLUMN_PHOTO_THUMBNAIL_URL = "PHOTO_THUMBNAIL_URL";
        public static final String COLUMN_PHOTO_THUMBNAIL_LOCAL = "PHOTO_THUMBNAIL_LOCAL";

        public static final String CREATE =
                "CREATE TABLE " + TABLE_NAME + " (" +
                        COLUMN_ID + " INTEGER PRIMARY KEY, " +
                        COLUMN_NAME + " TEXT NOT NULL, " +
                        COLUMN_ITEMS + " TEXT, " +
                        COLUMN_PHOTO_URL + " TEXT, " +
                        COLUMN_PHOTO_THUMB_URL + " TEXT, " +
                        COLUMN_PHOTO_LOCAL + " TEXT, " +
                        COLUMN_PHOTO_THUMB_LOCAL + " TEXT, " +
                        COLUMN_PHOTO_THUMBNAIL_URL + " TEXT, " +
                        COLUMN_PHOTO_THUMBNAIL_LOCAL + " TEXT" +
                        " ); ";

        public static ContentValues toContentValues(Set set) {
            ContentValues values = new ContentValues();
            values.put(COLUMN_ID, set.getId());
            values.put(COLUMN_NAME, set.getName());

            List<Integer> items = set.getItems();
            if (items != null && !items.isEmpty()) {
                Gson gson = new Gson();
                Type type = new TypeToken<List<Integer>>() {}.getType();
                String json = gson.toJson(items, type);
                values.put(COLUMN_ITEMS, json);
            }
            values.put(COLUMN_PHOTO_URL, set.getPhoto());
            values.put(COLUMN_PHOTO_THUMB_URL, set.getPhotoThumb());
            values.put(COLUMN_PHOTO_THUMBNAIL_URL, set.getPhotoThumbnail());
            /*
            values.put(COLUMN_PHOTO_LOCAL, "");
            values.put(COLUMN_PHOTO_THUMB_LOCAL, "");
            values.put(COLUMN_PHOTO_THUMBNAIL_LOCAL, "");
            */
            return values;
        }

        public static Set parseCursor(Cursor cursor) {
            Set set = new Set();
            set.setId(cursor.getInt(cursor.getColumnIndexOrThrow(COLUMN_ID)));
            set.setName(cursor.getString(cursor.getColumnIndexOrThrow(COLUMN_NAME)));

            String items = cursor.getString(cursor.getColumnIndexOrThrow(COLUMN_ITEMS));
            if (items != null && !items.isEmpty()) {
                Gson gson = new Gson();
                Type type = new TypeToken<List<Integer>>() {}.getType();
                List<Integer> fromJson = gson.fromJson(items, type);
                set.setItems(fromJson);
            }
            set.setPhoto(cursor.getString(cursor.getColumnIndexOrThrow(COLUMN_PHOTO_URL)));
            set.setPhotoThumb(cursor.getString(cursor.getColumnIndexOrThrow(COLUMN_PHOTO_THUMB_URL)));
            set.setPhotoThumbnail(cursor.getString(cursor.getColumnIndexOrThrow(COLUMN_PHOTO_THUMBNAIL_URL)));

            return set;
        }

    }
}
