package hikapro.com.backpack.presenter;

import android.content.Context;
import android.os.Bundle;
import android.support.v7.widget.DefaultItemAnimator;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.ViewGroup;
import android.widget.Toast;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;

import hikapro.com.backpack.R;
import hikapro.com.backpack.model.ItemModel;
import hikapro.com.backpack.model.Model;
import hikapro.com.backpack.model.entities.Item;
import hikapro.com.backpack.model.entities.Set;
import hikapro.com.backpack.presenter.adapters.CategoryListAdapter;
import hikapro.com.backpack.presenter.adapters.ItemListAdapter;
import hikapro.com.backpack.view.View;
import hikapro.com.backpack.view.recycler.CategoryViewHolder;
import hikapro.com.backpack.view.recycler.ItemViewHolder;

/**
 * Created by tariel on 20/04/16.
 */
public class ItemListPresenter implements Presenter.ItemList {

    private static final String BUNDLE_SET_LIST_KEY = "BUNDLE_SET_LIST_KEY";

    private WeakReference<View.ItemList>    view;
    private Model.Item                      model;
    private Set                             set;
    private CategoryListAdapter             categoryListAdapter;

    public ItemListPresenter() {
        this.categoryListAdapter    = new CategoryListAdapter(this);
    }

    // life cycle -->

    @Override
    public void onDestroy(boolean isChangingConfiguration) {
        view = null;
        model.onDestroy(isChangingConfiguration);
        if ( !isChangingConfiguration ) {
            model = null;
        }
    }
    @Override
    public android.view.View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        if (savedInstanceState != null)
            set = (Set) savedInstanceState.getSerializable(BUNDLE_SET_LIST_KEY);
        android.view.View view = inflater.inflate(R.layout.fragment_item_list, container, false);
        LinearLayoutManager llm = new LinearLayoutManager(getActivityContext());
        RecyclerView mainRecycler = (RecyclerView) view.findViewById(R.id.categories_main_recycler);
        mainRecycler.setLayoutManager(llm);
        mainRecycler.setAdapter(categoryListAdapter);
        mainRecycler.setItemAnimator(new DefaultItemAnimator());
        model.executeQuery();
        return view;
    }
    @Override
    public void onSaveInstanceState(Bundle outState) {
        if (set != null)
            outState.putSerializable(BUNDLE_SET_LIST_KEY, set);
    }

    // life cycle <--

    // recycler -->

    @Override
    public CategoryViewHolder createViewHolderCategory(ViewGroup parent, int viewType) {
        CategoryViewHolder viewHolder;
        android.view.View v = LayoutInflater.from(parent.getContext()).inflate(R.layout.section,
                parent, false);
        viewHolder = new CategoryViewHolder(v);
        return viewHolder;
    }
    @Override
    public void bindViewHolderCategory(CategoryViewHolder holder, int position) {
        final hikapro.com.backpack.model.entities.Category category = model.getCategoryByPosition(position);
        holder.sectionText.setText(category.getName());
        LinearLayoutManager llm = new LinearLayoutManager(getActivityContext());
        holder.itemsRecycler.setLayoutManager(llm);
        ItemListAdapter itemListAdapter = new ItemListAdapter(this);
        itemListAdapter.setCategoryId(category.getId());
        holder.itemsRecycler.setAdapter(itemListAdapter);
        holder.itemsRecycler.setItemAnimator(new DefaultItemAnimator());
        categoryListAdapter.addItemAdapter(itemListAdapter);
    }

    @Override
    public ItemViewHolder createViewHolderItem(ViewGroup parent, int viewType) {
        ItemViewHolder viewHolder;
        android.view.View v = LayoutInflater.from(parent.getContext()).inflate(R.layout.item,
                parent, false);
        viewHolder = new ItemViewHolder(v);
        return viewHolder;
    }
    @Override
    public void bindViewHolderItem(final ItemViewHolder holder, final int position, int categoryId) {

        hikapro.com.backpack.model.entities.Item item = model.getItemByPosition(categoryId, position);
        holder.title.setText(item.getName());
        holder.id = item.getId();
        holder.title.setOnClickListener(new android.view.View.OnClickListener() {
            @Override
            public void onClick(android.view.View view) {
                clickItem(holder.id);
            }
        });
    }
    @Override
    public int getItemsCount(int categoryId) {
        return model.getItemsCount(categoryId);//TODO category Id
    }
    @Override
    public int getCategoriesCount() {
        return model.getCategoriesCount();
    }


    // recycler <--

    // process -->

    @Override
    public void notifyDataSetChanged() {
        categoryListAdapter.notifyDataSetChanged();
        categoryListAdapter.notifyItemAdapters();
    }

    @Override
    public void clickItem(int itemId) {
        Item item = model.findItem(itemId);
        if (item != null)
            getView().showItemDetail(item);
        else
            showMessage(String.format("Item with Id %d is not found.", itemId));
    }
    // process <--

    // other impl -->

    @Override
    public void setView(View.ItemList view) {
        this.view = new WeakReference<>(view);
        this.set = getView().getSet();
    }
    @Override
    public void setModel(Model.Item model) {
        this.model = model;
    }
    @Override
    public Context getAppContext() {
        try {
            return getView().getAppContext();
        } catch (NullPointerException e) {
            return null;
        }
    }
    @Override
    public Context getActivityContext() {
        try {
            return getView().getActivityContext();
        } catch (NullPointerException e) {
            return null;
        }
    }
    @Override
    public void showMessage(String message) {
        Toast.makeText(getView().getAppContext(), message, Toast.LENGTH_SHORT).show();
    }

    // other impl <--

    private View.ItemList getView() throws NullPointerException {
        if ( view != null )
            return view.get();
        else
            throw new NullPointerException("View is unavailable");
    }


    @Override
    public Set getCurrentSet() {
        return set;
    }
}
