package hikapro.com.backpack.presenter;

import android.content.Context;
import android.os.Bundle;
import android.support.v4.view.MotionEventCompat;
import android.support.v7.widget.DefaultItemAnimator;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.helper.ItemTouchHelper;
import android.view.DragEvent;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.ViewGroup;
import android.widget.Toast;

import java.lang.ref.WeakReference;
import java.util.Collections;

import hikapro.com.backpack.R;
import hikapro.com.backpack.model.SetModel;
import hikapro.com.backpack.model.Model;
import hikapro.com.backpack.model.entities.Set;
import hikapro.com.backpack.presenter.adapters.SetListAdapter;
import hikapro.com.backpack.presenter.adapters.helper.OnStartDragListener;
import hikapro.com.backpack.presenter.adapters.helper.SimpleItemTouchHelperCallback;
import hikapro.com.backpack.view.View;
import hikapro.com.backpack.view.fragments.SetListFragment;
import hikapro.com.backpack.view.recycler.SetViewHolder;

/**
 * Created by tariel on 20/04/16.
 */
public class SetListPresenter implements Presenter.SetList {

    private WeakReference<View.SetList> view;
    private Model.Set                   model;
    private SetListAdapter              adapter;


    //GLM_add_resources_SetList
    private OnStartDragListener mDragStartListener;
    private ItemTouchHelper mItemTouchHelper;

    public SetListPresenter() {
        this.adapter    = new SetListAdapter(this);
    }

    // life cycle -->


    //GLM_add_resources_SetList
    @Override
    public void onItemDismiss(int position) {
        model.getSet().remove(position);
        adapter.notifyItemRemoved(position);
    }

    //GLM_add_resources_SetList
    @Override
    public boolean onItemMove(int fromPosition, int toPosition) {
        Collections.swap(model.getSet(), fromPosition, toPosition);
        adapter.notifyItemMoved(fromPosition, toPosition);
        return true;
    }

    @Override
    public void onDestroy(boolean isChangingConfiguration) {
        view = null;
        model.onDestroy(isChangingConfiguration);
        if ( !isChangingConfiguration ) {
            model = null;
        }
    }
    @Override
    public android.view.View onCreateView(LayoutInflater inflater, ViewGroup container,
                                          Bundle savedInstanceState) {
        android.view.View view = inflater.inflate(R.layout.fragment_set_list, container, false);
        LinearLayoutManager llm = new LinearLayoutManager(getActivityContext());
        RecyclerView setRecycler = (RecyclerView) view.findViewById(R.id.set_recycler);
        setRecycler.setLayoutManager(llm);
        setRecycler.setAdapter(adapter);
        setRecycler.setItemAnimator(new DefaultItemAnimator());
        model.executeQuery();

        //GLM_add_resources_SetList
        ItemTouchHelper.Callback callback = new SimpleItemTouchHelperCallback(adapter);
        mItemTouchHelper = new ItemTouchHelper(callback);
        mItemTouchHelper.attachToRecyclerView(setRecycler);

        return view;
    }

    // life cycle <--

    // recycler -->

    @Override
    public SetViewHolder createViewHolder(ViewGroup parent, int viewType) {
        SetViewHolder viewHolder;
        android.view.View v = LayoutInflater.from(parent.getContext()).inflate(R.layout.set_card,
                parent, false);
        viewHolder = new SetViewHolder(v);
        return viewHolder;
    }
    @Override
    public void bindViewHolder(final SetViewHolder holder, int position) {

        final hikapro.com.backpack.model.entities.Set set = model.getSetByPosition(position);
        holder.textView.setText(set.getName());
        holder.cardView.setOnClickListener(new android.view.View.OnClickListener() {
            @Override
            public void onClick(android.view.View v) {
                if (getView() != null)
                    getView().showItemList(set);
                else
                    showMessage("There is no view in presenter");
            }
        });


        //GLM_add_resources_SetList
        holder.cardView.setOnLongClickListener(new android.view.View.OnLongClickListener() {
            @Override
            public boolean onLongClick(android.view.View v) {

                mDragStartListener.onStartDrag(holder);
                return false;
            }
        });
    }
    @Override
    public int getSetsCount() {
        return model.getSetsCount();
    }

    // recycler <--

    // process -->

    @Override
    public void clickSet(int setId) {
        Set set = model.findSet(setId);
        if (set != null) {
            getView().showItemList(set);
        } else {
            showMessage("Set not found!");
        }
    }
    @Override
    public void notifyDataSetChanged() {
        adapter.notifyDataSetChanged();
    }

    // process <--

    // other impl -->

    @Override
    public Context getAppContext() {
        try {
            return getView().getAppContext();
        } catch (NullPointerException e) {
            return null;
        }
    }
    @Override
    public Context getActivityContext() {
        try {
            return getView().getActivityContext();
        } catch (NullPointerException e) {
            return null;
        }
    }
    @Override
    public void setView(View.SetList view) {
        this.view = new WeakReference<>(view);
        this.mDragStartListener = getView().getOnStartDragListener();
    }
    @Override
    public void setModel(Model.Set model) {
        this.model = model;
    }
    @Override
    public void showMessage(String message) {
        Toast.makeText(getView().getAppContext(), message, Toast.LENGTH_SHORT).show();
    }

    // other impl <--

    private View.SetList getView() throws NullPointerException {
        if ( view != null )
            return view.get();
        else
            throw new NullPointerException("View is unavailable");
    }

    //GLM_add_resources_SetList
    @Override
    public void onStartDrag(RecyclerView.ViewHolder viewHolder) {
        mItemTouchHelper.startDrag(viewHolder);
    }
}
